c physical property arrays in plib.f routines:

c the basic properties from vertex are in the arrays props, ptot, ptot1

c props(np,k5,k2) for the k5'th compund of the k2'th assemblage,
c np is the physical property as follows:

c    2 - atwt (g)
c    3 - molar enthalpy (J)
c    4 - volume (J/bar)
c    5 - Cp (J/K)
c    6 - dV/dT (J/bar/K)
c    7 - dV/dP (J/bar/bar)
c    8 - d2V/dT2
c    9 - (dV/dT)/dP
c   10 - d2V/dP2
c   11 - d3V/dT3
c   12 - shear modulus
c   13 - temperature derivative of the shear modulus
c   14 - pressure derivative of the shear modulus

c iflu(k2) = 1 if fluid present, otherwise = 0.

c additionally we have the following proportions and bulk properties:

c  ptot(1,k2)  - aggregate total moles
c  ptot(2,k2)  - aggregate total weight

c  ptot1(1,k2) - aggregate-fluid total moles
c  ptot1(2,k2) - aggregate-fluid total weight


c from these routine seismo computes the following properties as 
c a function of pressure and temperature:

c in array pphi(j,k5) for each cpd:

c 1 - vphi(k5)   - cpd volume 
c 2 - ephi(k5)   - cpd enthalpy 
c 3 - grunp(k5)  - cpd gruenesien ratio
c 4 - bulkmp(k5) - cpd adiabatic bulk modulus
c 5 - mup(k5)    - cpd shear modulus
c 6 - vel0p(k5)  - cpd sound velocity 
c 7 - velsp(k5)  - cpd s-wave velocity
c 8 - velpp(k5)  - cpd p-wave velocity
c 9 - vp/vs
c10 - rhop(k5)   - cpd density
c11 - vfract(k5) - cpd volume fraction
c12 - cp
c13 - alpha
c14 - beta 

c in arrays psys(j) and psys1(j)
c (vsys, esys, cp, alpha, beta, rho are calculated by arithmetic mean, the
c values of cp alpha and beta used for seismic props are computed by VRH
c averaging).

c 1 vsys       - sys molar volume 
c 2 esys       - sys molar enthalpy 
c 3 grun       - sys gruenesien ratio
c 4 bulkm      - sys adiabatic bulk modulus
c 5 mu         - sys shear modulus
c 6 vel0       - sys sound velocity 
c 7 velp       - sys p-wave velocity
c 8 vels       - sys s-wave velocity
c 9 vp/vs      
c 10 rsys      - sys density
c 11 csys      - sys specific heat capacity
c 12 cp
c 13 alpha
c 14 beta 

c additionally the total (cpd sum) for a solution can be obtained
c with routine vmodes.

	subroutine amiin (x,y,iam)

c amiin - routine to check if a (x,y) point lies inside a polygon.
c         copy of katja's check inside, assumes last point duplicates
c         first point.

c         alternatively (icopt=5) amiin identifies the assemblage 
c         associated with a grid point.

c         iam - is the polygon id if x-y is bounded otherwise 0.

      implicit real (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'

      parameter (l3=l2+2)

      common/ poly /px(l5,k2),py(l5,k2),npts(k2),iptoa(k2),ibd(k2),npoly
     *      / cst309 /dx,dy,igrd(l7,l7),loopx,loopy
     *      / vs     /vmin(l3),vmax(l3),iv
      common/ basic  /n4,ifont,icopt,jwidth,iop0

      if (icopt.eq.5) then 
c                                     find node associated with condition
         delx = x-vmin(1)
         ix = delx/dx 
         res = delx - ix*dx
         if (res.gt.0.5*dx) ix = ix + 1
         ix = ix + 1

         dely = y-vmin(2)
         jy = dely/dy 
         res = dely - jy*dy
         if (res.gt.0.5*dy) jy = jy + 1
         jy = jy + 1

         iam = igrd(ix,jy)

         goto 99

      end if 
c                                     do for each polygon:
      do i = 1, npoly

         if (ibd(i).eq.1) then 
         
            igo = 0

            do j = 1, npts(i) - 1

               if ((y.le.py(j,i).and.y.gt.py(j+1,i)).or. 
     *             (y.gt.py(j,i).and.y.le.py(j+1,i))) then 
	 
                  if (( (py(j+1,i)-py(j,i))*(x-px(j,i))
     *                - (px(j+1,i)-px(j,i))*(y-py(j,i)) )
     *                * (py(j+1,i)-py(j,i)) .lt. 0e0) igo = iabs(igo-1)

               end if 
            end do 
c                                      odd number of flips
c                                      the point is inside
            if (igo.eq.1) then 
               iam = iptoa(i)
               goto 99
            end if 
         end if 
      end do 

      iam = 0	

99    end 

      subroutine pplinp (fname)

c pplinp - read polygon file

      implicit real (a-g,o-z),integer (h-n)
 
      character*100 fname,n5name
 
      include 'perplex_parameters.h'

      common / basic /n4,ifont,icopt,jwidth,iop0
     */ poly  /px(l5,k2),py(l5,k2),npts(k2),iptoa(k2),ibd(k2),npoly

      n5 = n4 + 1
c                                    open polygon file = 'p' + plot file name.
      n5name = fname 
      call inblnk (n5name,'p')
      open (n5,file=n5name,status='old')
      read (n5,*,end=90) npoly,ncoor

      if (npoly.gt.k2) then 
         write (*,1000) npoly,k2
         stop
      else if (ncoor.gt.l5) then 
         write (*,1010) ncoor,l5
         stop
      end if 

      do i = 1, npoly
         read (n5,*,end=90) npts(i),iptoa(i),ibd(i)
         npts(i) = npts(i)/2
         read (n5,*) (px(j,i), py(j,i), j = 1, npts(i))
      end do 

      goto 99

90    write (*,1020) n5name,fname
      stop

1000  format (/,'Too many polygons (',i5,') increase parameter k2 (',
     *        i5,' and recompile WERAMI',/)
1010  format (/,'Too many coordinates in a polygon (',i5,') increase',
     *          ' parameter k2 (',i5,' and recompile WERAMI',/)
1020  format (/,'Inconsistent polygon file:',/,a,/
     *          'You have not run POLYGON since generating ',/,a,/,
     *          'with VERTEX, or POLYGON terminated abnormally.',/)
99    end 

      subroutine plinp (tmin,tmax,pmin,pmax)
 
c plinp - subroutine to read x-y plot file header.

      implicit real (a-g,o-z),integer (h-n)
 
      character*8 tname*162,vname,names,sname*10,fname*100,cname*5
 
      include 'perplex_parameters.h'

      double precision dblk(3,k5)

      parameter (l3=l2+2)

      common/ vs    /vmin(l3),vmax(l3),iv/ vsa /vname(l3)
     *      / cst319 /iemod(k2),kmod  
      common/ cst8  /names(k1)/ csta7 /sname(h9)/ phase /ikp(k1)
     *      / excl1 /ixct(3),iphct,isoct,iex(50,3),jex(50,3),ict(3)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / basic /n4,ifont,icopt,jwidth,iop0
     *      / csta4 /cname(k5)/ cst20 /comps(k1,k5),atwt(k5),icomp
     *      / cst309 /dx,dy,igrd(l7,l7),loopx,loopy
     *      / cst310 /tname(4)/ cst315 /jcont
     *      / cst316 /c0,c1,c2,c3,c4,iind,idep

      jcont = 0 
      read (n4,*,err=91) icopt
      read (n4,*) iphct, isoct, icomp
      read (n4,*) jbulk, ivol, icp, jvol, kmod
      if (icomp.ne.0) then
         read (n4,1040) (cname(i), i = 1, icomp)
         read (n4,*) (atwt(i), i = 1, icomp)
      end if 
      if (iphct.ne.0) then
         read (n4,1010) (names(i),i = 1, iphct)
         read (n4,*) ((comps(i,j), j= 1, icomp), i = 1, iphct)
         read (n4,*) (ikp(i), i = 1, iphct)
      end if
      if (isoct.ne.0) read (n4,1030) (sname(i), i = 1, isoct) 
      read (n4,1000) tname
      read (n4,*) iv,(jv(i),i=1,iv),jpt
      read (n4,*) jcont,iind,idep,c0,c1,c2,c3,c4
      read (n4,*) (vmax(i),vmin(i),i=1,iv)
      read (n4,1000) (vname(i),i=1,iv)

      if (icopt.eq.5) then 
c                                       compositions
         if (jcont.gt.0) read (n4,*) 
     *                   ((dblk(i,j),j=1,jbulk),i=1,jcont+1)
c                                       grid plot file
         read (n4,*) loopx, loopy
 
         if (loopx.eq.1) then
            write (*,1060) vmin(1)
            vmax(1) = vmin(1)+0.001
            dx = 0.002
         else 
            dx = (vmax(1) - vmin(1))/float(loopx-1)
         end if 

         dy = (vmax(2) - vmin(2))/float(loopy-1)
c                                       decompress the grid data
         do i = 1, loopx
            jst = 1
            do while (jst.le.loopy)
               read (n4,*) irep, kd
               jend = jst + irep 
               do j = jst, jend
                  igrd(i,j) = kd
               end do 
               jst = jend + 1
            end do 
         end do 
      end if           
c                                       for polygon:
      jcopt = icopt 

      tmin = vmin(1)
      tmax = vmax(1)
      pmin = vmin(2)
      pmax = vmax(2)

      goto 99

91    write (*,1050) fname
      stop

1000  format (a)
1010  format (10a8)
1030  format (8a10)
1040  format (20a5)
1050  format (/,'File:',/,a,/,' is not formatted correctly.',/)
1060  format (/,'** warning ** you are plotting a 1-dimensional grid ',
     *        'as a 2-dimensional',/,'plot. The width of the X-axis ',
     *        'is arbitrary, the true value for the X-axis',/,
     *        'variable is: ',g12.6,/)
1070  format (/,'Enter the VERTEX plot file name: ')

99    end

c-------------------------------------------------------------------
      subroutine bplinp (fname,kpoly)

c subroutine to read bulk composition data file for pseudosections.

c props(np,k5,k2) for the k5'th compund of the k2'th assemblage,
c np is the physical property as follows:

c    2 - atwt (g)
c    3 - molar enthalpy (J)
c    4 - volume (J/bar)
c    5 - Cp (J/K)
c    6 - dV/dT (J/bar/K)
c    7 - dV/dP (J/bar/bar)
c    8 - dCp/dT
c    9 - dCp/dP
c   10 - d2V/dT2
c   11 - (dV/dT)/dP
c   12 - (dV/dP)/dT
c   13 - dV2/dP2
c   14 - shear modulus
c   15 - temperature derivative of the shear modulus
c   16 - pressure derivative of the shear modulus

c iflu(k2) = 1 if fluid present, otherwise = 0.

c additionally we have the following proportions and bulk properties:

c  ptot(1,k2)  - aggregate total moles
c  ptot(2,k2)  - aggregate total weight
c  ptot(4,k2)  - aggregate total volume
c  ptot1(1,k2) - aggregate-fluid total moles
c  ptot1(2,k2) - aggregate-fluid total weight
c  ptot1(4,k2) - aggregate-fluid total volume

c from these we can define 

c props(1,i,jd)/ptot(1,jd)                 - compound mol fraction 
c props(1,i,jd)*props(2,i,jd)/ptot(2,jd)   - compound wt fraction
c props(1,i,jd)*props(4,i,jd)/ptot(4,jd)   - compound volume fraction


      implicit real (a-g,o-z),integer (h-n)

      character*100 fname, n5name
 
      include 'perplex_parameters.h'

      parameter (l3=l2+2)

      common/ basic /n4,ifont,icopt,jwidth,iop0
     *      / cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst20 /comps(k1,k5),atwt(k5),icomp
     *      / vs    /vmin(l3),vmax(l3),iv/ cst315 /jcont   
     *      / cst319 /iemod(k2),kmod 

      n5 = n4 + 1
      ibulk = 1
c                                    open assemblage information
c                                    file = 'b' + plot file name.
      n5name = fname 
      call inblnk (n5name,'b')
      open (n5,err=90,file=n5name,status='old')

10    read (n5,*,end=20) jd, ivar(ibulk), iap(ibulk), iflu(ibulk),
     *                   (idbk(i,ibulk), i = 1, jbulk)
      read (n5,*) (vr(i,ibulk), i = 1, iv - jcont)
      read (n5,*) ((props(j,i,ibulk), j = 1, 11), i = 1, jbulk)
      read (n5,*) (ptot(j,ibulk), j = 1, 7),btot(ibulk),gtot(ibulk),
     *            (fbulk(j,ibulk), j = 1, icomp)
      if (iflu(jd).eq.1) then 
         read (n5,*) (iflug(j,ibulk), j = 1, jbulk)
         read (n5,*) (ptot1(j,ibulk), j = 1, 7),
     *        btot1(ibulk),gtot1(ibulk),(fbulk1(j,ibulk), j = 1, icomp)
      end if 
 
      if (kmod.ne.0) then 
         read (n5,*) iemod(ibulk)
         if (iemod(ibulk).ne.0) read (n5,*) 
     *     ((props(j,i,ibulk), j= 12, 14), i = 1, jbulk)
      else 
         iemod(ibulk) = 0d0
      end if 

      ibulk = ibulk + 1
      goto 10

20    close (n5)

      ibulk = ibulk - 1
c                                    open polygon file = 'p' + plot file name.

      kpoly = ibulk 

      goto 99 

90    write (*,1000) n5name,fname
      stop

1000  format (/,'The bulk information file:',/,a,/,'corresponding to'
     *         ,' plot file:',/,a,/,'does not exist.',//)

99    end   
c-------------------------------------------------------------------
      subroutine calpr1 (x,y,jd,n3) 

c calprp - compute full vertex style print output from b-plot file.

      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
 
      character cname*5, pname(k5)*20, vname*8

      real x, y

      parameter (l3=l2+2)

      common/ vs    /vmin(l3),vmax(l3),iv/ vsa /vname(l3)
      common/ csta4  /cname(k5)/ cst20 /comps(k1,k5),atwt(k5),icomp
      common/ basic /n4,ifont,icopt,jwidth,iop0/ cst315 /jcont 
     *      / cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst317 /pphi(i7,k5),psys(i7),psys1(i7),sphi(i7,k5),
     *                idphi(k5),nphi
c                                      determine if x and/or y are 
c                                      pressure or temperature:
      call xytopt (x,y,p,t,dp,dt,jd)   
c                                      compute derivative props:                                    
      call seismo (p,t,dp,dt,jd)

      write (n3,1000) jd 
      write (n3,1120) vname(1), x, vname(2), y 
      if (iv.gt.2) write (n3,1120) (vname(i), vr(i-jcont,jd), i = 3, iv)
      write (n3,1020) (cname(i), i = 1, icomp)
  
      do i = 1, jbulk
         k = idbk(i,jd)
         call blktxt (pname(i),k)
         write (n3,1030) pname(i), props(1,i,jd)/ptot(1,jd)*1e2, 
     *                   props(1,i,jd)*props(2,i,jd)/ptot(2,jd)*1e2,
     *                   pphi(11,i)*1e2,(comps(k,l), l = 1, icomp)
      end do 

      write (n3,1160)
c                                     phase/system summary, normal thermo:
      do i = 1, jbulk
c                                     N, H, V, Cp, alpha, beta
         write (n3,1170) pname(i), props(2,i,jd),pphi(2,i),pphi(1,i),
     *                  (pphi(j,i), j = 12, 14), pphi(10,i)
      end do

      write (n3,1170) 'System              ',ptot(2,jd),psys(2),
     *                psys(1),psys(12),psys(13),psys(14)
      if (iflu(jd).eq.1) 
     *write (n3,1170) 'System - Fluid      ',ptot1(2,jd),psys1(2),
     *                psys1(1),psys1(12),psys1(13),psys1(14)

      write (n3,1190)
c                                     phase/system summary, seismic:
      do i = 1, jbulk
         write (n3,1200) pname(i), (pphi(j,i), j = 3, 8) 
      end do

      if (iflu(jd).eq.0) then 
c                                     no fluid is present:
         write (n3,1040)

         do i = 1, icomp

            write (n3,1110) cname(i),fbulk(i,jd), 
     *                      fbulk(i,jd)/gtot(jd)*1e2,
     *                      fbulk(i,jd)*atwt(i)/btot(jd)*1e2                            
         end do

         write (n3,1060) psys(10), psys(2)/psys(1)*1d5, 
     *                   psys(12)/psys(1)*1d5,(psys(j),j= 3, 8)

      else 
c                                     fluid is present
         write (n3,1080)

         do i = 1, icomp

            write (n3,1110) cname(i),fbulk(i,jd),
     *                  fbulk(i,jd)/gtot(jd)*1e2,
     *                  fbulk(i,jd)*atwt(i)/btot(jd)*1e2,
     *                  fbulk1(i,jd)/gtot1(jd)*1e2,
     *                  fbulk1(i,jd)*atwt(i)/btot1(jd)*1e2          
         end do

         write (n3,1060) psys(10), psys(2)/psys(1)*1d5, 
     *                   psys(12)/psys(1)*1d5,(psys(j),j= 3, 8)
         write (n3,1100) psys1(10), psys1(2)/psys1(1)*1d5, 
     *                   psys1(12)/psys(1)*1d5,(psys1(j),j= 3, 8)

      end if 

      write (n3,1070) 2, ivar(jd)

      write (n3,1180) vname(jpt(1)),vr(jpt(1)-jcont,jd),
     *                vname(jpt(2)),vr(jpt(2)-jcont,jd)

1000  format (' ---------',/,1x,'(',i5,') Stable phases at:')
1020  format (55x,'Molar Phase Compositions:',
     *        /,25x,'mol %',6x,'wt %',4x,'vol % (2)',3x,20(a,3x))
1030  format (1x,a20,3x,3(f6.2,4x),20(f6.2,2x))
1040  format (/,14x,'mol',7x,'mol %',6x,'wt %')
1050  format (/,'WARNING - negative mass indicates the bulk'
     *         ,' composition is',/,'inconsistent with the'
     *         ,' saturation constraints',/)
1060  format (/,'Density (kg/m3) (2) = ',f7.1,/, 
     *          'Specific Enthlpy (J/m3) (2) = ',g12.6,/,
     *          'Specific Heat Capacity (J/K/m3) (1) = ',g12.6,/,
     *          'Aggregate Grueneisen Ratio (2,3) = ',g12.6,/,
     *          'Aggregate Adiabatic Bulk Modulus (bar) (2,3) = ',g12.6,
     *        /,'Aggregate Shear Modulus (bar) (2,3,4) = ',g12.6,/,
     *          'Aggregate Sound Velocity (km/s) (2,3) = ',g12.6,/,
     *          'Aggregate P-Wave Velocity (km/s) (2,3,4) = ',g12.6,/,
     *          'Aggregate S-Wave Velocity (km/s) (2,3,4) = ',g12.6,/)
1070  format (/,'Variance (c-p+',i1,') = ',i2)
1080  format (/,16x,'Complete Assemblage',15x,'Solids Only',
     *        /,14x,'mol',7x,' mol %',6x,'wt %',9x,' mol %',6x,'wt %',/)
1100  format (/,'Solid Density (kg/m3) (2) = ',f7.1,/, 
     *          'Solid Specific Enthlpy (J/m3) (2) = ',g12.6,/,
     *          'Solid Specific Heat Capacity (J/K/m3) (1) = ',g12.6,/,
     *          'Aggregate Solid Grueneisen Ratio (2,3) = ',g12.6,/,
     *          'Aggregate Solid Adiabatic Bulk Modulus (bar) (2,3) = ',
     *  g12.6,/,'Aggregate Solid Shear Modulus (bar) (2,3,4) = ',g12.6,/
     *         ,'Aggregate Solid Sound Velocity (km/s) (2,3) = ',g12.6,/
     *         ,'Aggregate Solid P-Wave Velocity (km/s) (2,3,4) = ',
     *  g12.6,/,'Aggregate Solid S-Wave Velocity (km/s) (2,3,4) = ',
     *                                                          g12.6,/)
1110  format (1x,a8,2x,2(f8.2,3x,2(f6.2,4x),5x))
1120  format (29x,a8,' = ',g12.6)
1130  format (20(i5,1x))
1140  format (12(f6.2,1x))
1150  format (12(g14.6,1x))
1160  format (/,25x,'Molar Properties (2):'
     *        /,27x,'N(g)',8x,'H(J)',5x,'V(J/bar)',6x,'Cp(J/K)'
     *         ,6x,'Alpha(1/K)',2x,'Beta(1/bar)',2x,'Density(kg/m3)')
1170  format (1x,a20,1x,f9.2,3x,12(g12.5,1x))
1180  format (/,'Notes:',/,
     *         '1 - Computed at reference condition (',a8,'=',f6.0,
     *         ', ',a8,'=',f5.0,').',/,
     *         '2 - Computed from linearized equation of state',
     *         ' relative to the reference condition.',/,
     *         '3 - Computed by Voight-Reuss-Hill scheme (e.g., Bina ',
     *         '& Helfrich 1992).',/,
     *         '4 - If zero, shear modulus data is not in the data ',
     *         'base.',/,'---------',/)
1190  format (/,25x,'Seismic (Molar) Properties(2):'
     *        /,24x,'Gruneisen',7x,'Ks(bar)',7x,'Mu(bar)',
     *        4x,'V0(km/s)',5x,'Vp(km/s)',5x,'Vs(km/s)')
1200  format (1x,a20,3x,12(g12.5,1x))

      end 

      subroutine xytopt (x,y,p,t,dp,dt,jd)
c------------------------------------------------------------------
c this is a routine to determine if the x-y variables of a section
c are pressure and temperature and to calculate the appropriate
c displacements dp and dt from the reference pressure temperature
c conditions for polygon properties. this is sloppy and could be 
c avoided by using the pointer system in vertex
c------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (l3=l2+2)
 
      common/ cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / vs    /vmin(l3),vmax(l3),iv/ cst315 /jcont

      dt = 0e0
      dp = 0e0
      p = vmin(jpt(1))
      t = vmin(jpt(2))

      if (jpt(1).eq.1) then
         dp = x - vr(jpt(1)-jcont,jd)
         p = x
      else if (jpt(1).eq.2) then
         dp = y - vr(jpt(1)-jcont,jd)
         p = y 
      end if

      if (jpt(2).eq.1) then
         dt = x - vr(jpt(2)-jcont,jd)
         t = x
      else if (jpt(2).eq.2) then
         dt = y - vr(jpt(2)-jcont,jd)
         t = y 
      end if  
    
      end 


      subroutine blktxt (pname,id)
c-----------------------------------------------------------------------
c subprogram to write a text labels for bulk composition output 
c-----------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(20)*1, fname*10, pname*20

      common/ cst8  /names(k1)/ csta7 /fname(h9)/ phase /ikp(k1)    
c                             dump the names into the array text
         if (ikp(id).eq.0) then
c                             simple compound:
            iend = 8
            read (names(id),1000) (text(j), j = 1, iend)
            call ftext1 (text,iend)
            write (pname,1000) (text(j), j = 1, iend)
         else  
c                             solution phases:
            iend = 20
            read (fname(ikp(id)),1000) (text(j), j = 1, 10)
            text(11) = '('
            read (names(id),1000) (text(j), j = 12, 19)
            text(20) = ')'
            call ftext1 (text,iend)
            write (pname,1000) (text(j), j = 1, iend)
         end if 

1000  format (20a1)

      end 

      subroutine ftext1 (text,iend)
c-----------------------------------------------------------------------
c subprogram to filter blanks from text 
c-----------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      character*1 text(20)

      itic = 0 

      do i = 1, iend

         if (text(i).ne.' ') then 
            itic = itic + 1
            text(itic) = text(i)
         end if 

      end do 

      iend = itic 
 
      end

      subroutine rerror (ier,*)
c---------------------------------------------------------------------
c rerror - routine to check for errors during list directed i/o
 
      implicit real (a-g,o-z),integer (h-n)
c---------------------------------------------------------------------
 
      if (ier.eq.0) then
         return
      else
         write (*,1000)
         ier = 0
         return 1
      end if
 
1000  format (/,'Your input is incorrect, probably you are using ',
     *        'a character where',/,'you should be using a number ',
     *        'or vice versa, try again...',/)
 
      end

c---------------------------------------------------------------
      subroutine rnam1 (iex,jex)

      implicit real (a-g,o-z),integer (h-n)

      character*10 xnam

      iex = 0
      jex = 0

110   write (*,1040) 
      read (*,1020) xnam

      call matchi (xnam,iex,jex)

      if (iex.eq.-1) then
         write (*,1100) xnam
         goto 110
      end if

      if (iex.eq.0) iex = -1

1020  format (a)
1040  format (/,'Enter solution or compound name (left justified): ')
1100  format (/,'No such entity as ',a,', try again: ')

      end

      subroutine matchi (unnown,itis,icpd)
c----------------------------------------------------------------
 
c matchi - subroutine to determine if the string unnown is a valid
c          solution or compound name.
 
c   itis = 0 if compound, = ikp if solution, and =-1 if invalid
c   icpd = the id of the compound if itis = 0

      implicit real (a-g,o-z),integer (h-n)
 
      character*10 sname, names*8, unnown
 
      include 'perplex_parameters.h'
 
      common/ excl1 /ixct(3),iphct,isoct,iex(50,3),jex(50,3),ict(3)
      common/ cst8 /names(k1)/ csta7 /sname(h9)/ phase /ikp(k1)
 
      if (isoct.ne.0) then
         do i = 1, isoct
            if (unnown.eq.sname(i)) then
               itis = i
               goto 99
             end if
         end do
      end if
 
      do i = 1, iphct
         if (unnown.eq.names(i)) then
            itis = 0
            icpd = i
            goto 99
         end if
      end do 
 
      itis = -1
99    end

      subroutine chsprp (lop,icx,jcx,icps,rcps)
c----------------------------------------------------------------
c chsprp asks the user to choose a property for contouring:
c   lop  - flag indicating the property chosen
c   icx  - if lop = 6, the component chosen
c   icx  - if lop > 6, the identity of the solution chosen,
c          icx = -1 if a solution is not chosen.
c   jcx  - if lop = 7, the identity of the compound chosen,
c          if a solution is not chosen.
c   jcx  - if lop = 8, the number of components to define the
c          the phase composition.
c   icps - if lop = 8, the indices of the components. 
c   jflu - 0 include fluids for bulk props, else 1.

c 1                 Bulk specific enthalpy (J/m3)',
c 2                 Bulk density (kg/m3)',
c 3                'Bulk specific Heat capacity (J/K/m3)',
c 4                'Bulk expansivity (1/K, for volume)',
c 5                'Bulk compressibility (1/bar, for volume)',
c 6                'Weight percent of a component',
c 7                'Mode (Vol %) of a compound or solution',
c 8                'Composition of a solution'
c 9                 Grueneisen thermal ratio',
c 10               'Adiabatic bulk modulus (bar)',
c 11               'Shear modulus (bar)'
c 12                Sound velocity (km/s)
c 13               'P-wave velocity (km/s)',
c 14                S-wave velocity (km/s)',
c 15                Vp/Vs
c----------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (l3=l2+2)

      character*5 cname
 
      common/ csta4 /cname(k5)/ cst20 /comps(k1,k5),atwt(k5),icomp
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst320 /jflu

      real rcps(k7)

      integer icps(k7)

      character propty(15)*60, y*1

      save propty

      data propty/'Specific enthalpy (J/m3)',
     *            'Density (kg/m3)',
     *            'Specific Heat capacity (J/K/m3)',
     *            'Expansivity (1/K, for volume)',
     *            'Compressibility (1/bar, for volume)',
     *            'Weight percent of a component',
     *            'Mode (Vol %) of a compound or solution',
     *            'Composition of a solution',
     *            'Grueneisen thermal ratio',
     *            'Adiabatic bulk modulus',
     *            'Shear modulus (bar)',
     *            'Sound velocity (km/s)',
     *            'P-wave velocity (Vp, km/s)',
     *            'S-wave velocity (Vs, km/s)',
     *            'Vp/Vs'/
c-------------------------------------------
      jflu = 0
c                                choose property
5000  write (*,1050)

      do i = 1, 15
         write (*,1060) i,propty(i)
      end do 

      read (*,*,iostat=ier) lop
      call rerror (ier,*5000)

      if (lop.eq.7) then 
c                                modes:
c                                get the phase name
          call rnam1 (icx,jcx)
c                                ask if fluid should be included:
          if (ivol.ne.0) then 
             write (*,1120) 
             read (*,'(a)') y
             if (y.eq.'y'.or.y.eq.'Y') jflu = 1 
          end if 

      else if (lop.ne.6.and.lop.ne.8) then 
c                                if lop .ne. 6 or 8 ask if
c                                bulk property or a phase property
         write (*,1110) 
         read (*,'(a)') y
         if (y.ne.'y'.and.y.ne.'Y') then 
            icx = -1 
            jcx = -1 
c                                it's a bulk property, ask if fluid
c                                should be included:
            if (ivol.ne.0) then 
               write (*,1120) 
               read (*,'(a)') y
               if (y.eq.'y'.or.y.eq.'Y') jflu = 1 
            end if 

         else 
c                                get the phase name
            call rnam1 (icx,jcx)
            if (icx.lt.1) write (*,*) 
     *         'hey cowboy, that warnt no solution, try again'
         end if
 
      else if (lop.eq.6) then
c                                get the component to be contoured
5010     write (*,1000)
         write (*,1010) (i,cname(i),i = 1, icomp)
         read (*,*,iostat=ier) icx
         call rerror (ier,*5010)   
c                                ask if fluids included
         if (ivol.ne.0) then 
            write (*,1120) 
            read (*,'(a)') y
            if (y.eq.'y'.or.y.eq.'Y') jflu = 1 
         end if 

      else if (lop.eq.8) then
c                                get the solution identity
5005     call rnam1 (icx,jcx)
         if (icx.lt.1) then 
            write (*,*) 'hey cowboy, that warnt no solution, try again'
            goto 5005
         end if 
c                                get the composition to be contoured
         write (*,1100)
5020     write (*,1020)
         write (*,1010) (i,cname(i),i = 1, icomp)
         read (*,*,iostat=ier) icps(1),rcps(1)
         call rerror (ier,*5020) 
5030     write (*,1030) 
         read (*,*,iostat=ier) jcx
         call rerror (ier,*5030) 
         jcx = jcx + 1
         if (jcx.gt.1) then 
5040        write (*,1040)
            write (*,1010) (i,cname(i),i = 1, icomp)
            read (*,*,iostat=ier) (icps(i),rcps(i), i = 2, jcx)
            call rerror (ier,*5040)
            write (*,1080) (rcps(i),cname(icps(i)), i = 1, jcx)
         else 
            write (*,1070) (rcps(i),cname(icps(i)), i = 1, jcx)
         end if 
 
         write (*,1090)
         read (*,'(a)') y
         if (y.ne.'y') goto 5020
      end if 

1000  format (/,'Enter a component:')
1010  format (2x,i2,' - ',a5)
1020  format (/,'Component index and weighting factor for the',
     *        /,'compositions numerator?')
1030  format (/,'How many components in the denominator of the',
     *          ' composition?',/,'Enter zero to plot the numerator.')
1040  format (/,'Component indices and weighting factors for the', 
     *        /,'compositions denominator?')
1050  format (/,'Select a property:')
1060  format (3x,i2,' - ',a60)
1070  format (/,'The compositional variable is: ',f4.1,1x,a5)
1080  format (/,'The compositional variable is: ',f4.1,1x,a5,
     *        ' / Sum ',20(f4.1,1x,a5,1x))
1090  format ('Happy (y/n?)')
1100  format (/,'Compositions are defined as a ratio of the form:',/,
     *        10x,' w(j)*n(j) / Sum {w(i)*n(i), i = 1, c}',/,15x,
     *        ' n(j) = number of moles of component j',/,15x,
     *        ' w(j) = weighting factor of component j (usually 1)',/)
1110  format (/,'Calculate individual phase properties (y/n)?')
1120  format (/,'Include fluid in computation of aggregate ', 
     *          '(or modal) properties (y/n)?')

99    end

      subroutine polprp (prop,x,y)

      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common / cst84 /rcps(k7),icps(k7),icx,jcx,lop

c                             amiin gets iam, the pointer to the assemblage
c                             in the "b" file as a function of x-y coordinate
      call amiin (x,y,iam)  

      if (iam.eq.0) then 
         prop = -1e0
         write (*,1000) x, y
      else 
         call getprp (x,y,lop,iam,prop,icx,jcx,icps,rcps)
      end if 

1000  format ('Missing data at: ',2(g12.6,1x),' assigned value -1')
      end 

      subroutine getprp (x,y,lop,jd,prop,icx,jcx,icps,rcps)
c----------------------------------------------------------------
c getprp gets properties:

c   jd   - is the pointer to the assemblage
c   jflu - if 1, bulk properties include fluid phase; else 0

c   lop  - flag indicating the property chosen
c   icx  - if lop = 6, the component chosen
c   icx  - if lop > 6, the identity of the solution chosen,
c          icx = -1 if a solution is not chosen.
c   jcx  - if lop = 7, the identity of the compound chosen,
c          if a solution is not chosen.
c   jcx  - if lop = 8, the number of components to define the
c          the phase composition.
c   jcx  - if lop > 8 the identity of the compound if icx < 0,
c          if jcx and icx < 0, then bulk properties are to be 
c          computed.
c   icps - if lop = 8, the indices of the components. 
c   

c 1                 Specific enthalpy (J/m3)',
c 2                 Density (kg/m3)',
c 3                 Specific Heat capacity (J/K/m3)',
c 4                 Expansivity (1/K, for volume)',
c 5                 Compressibility (1/bar, for volume)
c 6                 Weight percent of a component
c 7                 Mode (Vol %) of a compound or solution',
c 8                 Composition of a solution'
c 9                 Adiabatic bulk modulus (bar)',
c 10                Grueneisen thermal ratio',
c 11                Sound velocity (km/s)
c 12                Shear modulus (bar)',
c 13                P-wave velocity (km/s)',
c 14                S-wave velocity (km/s)',
c 15                Vp/Vs
c----------------------------------------------------------------

      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (l3=l2+2)

      real rcps(k7)

      integer icps(k7)

      common/ cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)/ cst320 /jflu
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst20 /comps(k1,k5),atwt(k5),icomp/ phase /ikp(k1)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst317 /pphi(i7,k5),psys(i7),psys1(i7),sphi(i7,k5),
     *                idphi(k5),nphi 



      if (lop.eq.8) then 
c                                  a solution composition
         call scomp (jd,prop,icx,jcx,icps,rcps)
      else if (lop.eq.6) then 
c                                  wt % of component 
         if (iflu(jd).eq.0.or.(iflu(jd).eq.1.and.jflu.eq.1)) then 
c                                  include fluid
            prop = fbulk(icx,jd)*atwt(icx)/btot(jd)*1e2
         else 
c                                  exclude fluid
            prop = fbulk1(icx,jd)*atwt(icx)/btot1(jd)*1e2
         end if 
      
      else 
c                                  get p-t for thermo props:
         call xytopt (x,y,p,t,dp,dt,jd)
c                                  calculate phase and bulk props
         call seismo (p,t,dp,dt,jd)
c                                  according to props:
         if (lop.eq.7) then 
c                                  volumetric modes are a special case, vmodes
c                                  returns the total mode
            call vmodes (prop,p,t,lop,icx,jcx,jd)

            if (iflu(jd).eq.1.and.jflu.eq.0) then 
c                                  recompute the mode to exclude fluid
               prop = prop*psys(1)/psys1(1)
            end if 

         else if (icx.lt.0.and.jcx.lt.0) then 
c                                  a system property is  requested:
            if (iflu(jd).eq.0.or.(iflu(jd).eq.1.and.jflu.eq.1)) then 
c                                  if jflu = 0 the property is to include 
c                                  fluid (if present), i.e, use psys/ptot array:
               if (lop.eq.1) then 
c 1                                  Specific enthalpy (J/m3)
                  prop = psys(2)/psys(1)*1d5
               else if (lop.eq.2) then 
c 2                                  Density (kg/m3)
                  prop = psys(10)
               else if (lop.eq.3) then 
c 3                                  Specific Heat capacity (J/K/m3)
                  prop = psys(12)/psys(1)*1d5
               else if (lop.eq.4) then
c 4                                  Expansivity (1/K, for volume)
                  prop =  psys(13)
               else if (lop.eq.5) then 
c 5                                  Compressibility (1/bar, for volume)
                  prop =  psys(14)
               else         
                  prop = psys(lop-6)
               end if 
            else 
               if (lop.eq.1) then 
                  prop =  psys1(2)/psys1(1)*1d5
               else if (lop.eq.2) then 
                  prop = psys1(10)
               else if (lop.eq.3) then  
                  prop = psys1(12)/psys1(1)*1d5
               else if (lop.eq.4) then
                  prop =  psys1(13)
               else if (lop.eq.5) then 
                  prop =  psys1(14)
               else         
                  prop = psys1(lop-6)
               end if
            end if 
         else 
c                               we want a phase property:
             call vmodes (prop,p,t,lop,icx,jcx,jd)

         end if 

      end if                
 
99    end 

      subroutine scomp (jd,comp,icx,jcx,icps,rcps)
c-------------------------------------------------------------------
c function scomp returns composition of a solution phase in 
c assemblage jd. if the phase is a solution icx is its index (>0),
c otherwise jcx is its index. the index is determined by rnam1.
c ------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (l3=l2+2)

      real rcps(k7)

      integer icps(k7)
 
      common/ cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst20 /comps(k1,k5),atwt(k5),icomp/ phase /ikp(k1)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr

      comp = 0e0 
      totden = 0e0 

      do i = 1, jbulk

         if (ikp(idbk(i,jd)).eq.icx) then 
c                                           got the solution: 
c                                           numerator:
            comp = comp 
     *           + props(1,i,jd)*rcps(1)*comps(idbk(i,jd),icps(1))
c                                           denominator:
            do j = 2, jcx 
               totden = totden 
     *                + rcps(j)*props(1,i,jd)*comps(idbk(i,jd),icps(j))
            end do

         end if    

      end do 

      if (totden.ne.0e0) comp = comp / totden

      end

      function wtp (jcomp,jd)
c-------------------------------------------------------------------
c function wtp returns weight fraction of component jcomp in 
c assemblage jd.
c ------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
 
      common/ cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst20 /comps(k1,k5),atwt(k5),icomp

      if (iflu(jd).eq.0) then 
        
            wtp = fbulk(jcomp,jd)*atwt(jcomp)/btot(jd)*1e2                           
  
      else 
c                                   if fluid is present use 
c                                   solid-only totals:
            wtp = fbulk1(jcomp,jd)*atwt(jcomp)/btot(jd)*1e2  

      end if 

      end

      subroutine vmodes (prop,p,t,iprop,icx,jcx,jd) 
c-------------------------------------------------------------------
c function vmode returns a property of a phase in 
c assemblage jd. if the phase is a solution icx is its index (>0),
c otherwise jcx is its index. the index is determined by rnam1.

c the property is identified by the index iprop:

c 1                 enthalpy (J/m3)
c 2                 density (kg/m3)
c 3                 Heat capacity (J/K/m3)
c 4                 Expansivity (1/K, for volume)
c 5                 Compressibility (1/bar, for volume)
c 7                 Mode (Vol %) of a compound or solution
c 9                 Adiabatic bulk modulus (bar)
c 10                Grueneisen thermal ratio
c 11                Sound velocity (km/s)
c 12                Shear modulus (bar)
c 13                P-wave velocity (km/s)
c 14                S-wave velocity (km/s)
c 15                Vp/Vs
c------------------------------------------------------------------
      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (l3=l2+2)
 
      common/ cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst20 /comps(k1,k5),atwt(k5),icomp/ phase /ikp(k1)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst317 /pphi(i7,k5),psys(i7),psys1(i7),sphi(i7,k5),
     *                idphi(k5),nphi
     *      / cst320 /jflu

      prop = 0d0

      if (icx.gt.0) then 
c                                           we are looking for a solution
c                                           prop saved in sphi:
         do j = 1, nphi
            if (ikp(idphi(j)).eq.icx) then 
               if (iprop.eq.1) then
c                                           specific enthalpy
                  prop = sphi(2,j)/sphi(1,j)*1e5
               else if (iprop.eq.2) then 
c                                           density
                  prop = sphi(10,j)
               else if (iprop.eq.3) then 
c                                           specific cp 
                  prop = sphi(12,j)/sphi(1,j)*1e5
               else if (iprop.eq.4) then 
c                                           expansivity
                  prop = sphi(13,j)
               else if (iprop.eq.5) then 
c                                           compressibility
                  prop = sphi(14,j)
               else if (iprop.eq.7) then                           
c                                           mode (%)
                  if (jflu.eq.1.or.iflu(jd).eq.0) then
c                                           total mode:
                     prop = sphi(11,j)*1e2
                  else 
c                                           solid only mode:
                     prop = sphi(11,j)*1e2*psys(1)/psys1(1)
                  end if 
               else if (iprop.ge.9.and.iprop.le.15) then 
c                                           seismic props
                  prop = sphi(iprop-6,j)                 
               end if 
               goto 99 
            end if 
         end do 

      else
c                                           we are looking for a cpd in pphi
         do j = 1, jbulk
            id = idbk(j,jd)
            if (id.eq.jcx) then 
               if (iprop.eq.1) then
c                                           specific enthalpy
                  prop = pphi(2,j)/pphi(1,j)*1e5
               else if (iprop.eq.2) then 
c                                           density
                  prop = pphi(10,j)
               else if (iprop.eq.3) then 
c                                           specific cp 
                  prop = pphi(12,j)/pphi(1,j)*1e5
               else if (iprop.eq.4) then 
c                                           expansivity
                  prop = pphi(13,j)
               else if (iprop.eq.5) then 
c                                           compressibility
                  prop = pphi(14,j)
               else if (iprop.eq.7) then                           
c                                           mode (%)
                  if (jflu.eq.0.or.iflu(jd).eq.0) then
c                                           total mode:
                     prop = pphi(11,j)*1e2
                  else 
c                                           solid only mode:
                     prop = pphi(11,j)*1e2*psys(1)/psys1(1)
                  end if 
               else if (iprop.ge.9.and.iprop.le.15) then 
c                                           seismic props
                  prop = pphi(iprop-6,j)                 
               end if 
               goto 99 
            end if 
         end do 

      end if     

99    end

      subroutine inblnk (text,char)
c------------------------------------------------------------------- 
c inblnk - scan text to last '/' or '\' and insert char after.
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1, char*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = nchar,1,-1
         if (bitsy(i).eq.'/') goto 10
         bitsy(i+1) = bitsy(i)
      end do 

      i = 0

10    bitsy(i+1) = char

      write (text,1000) (bitsy(i), i = 1, nchar)
 
1000  format (400a1)

      end


c-------------------------------------------------------------------
      subroutine seismo (p,t,dp,dt,jd)  

c input:

c   p, t, dp, dt - conditions set by prior call to  xytopt 

c output:

c 1 - vphi(k5)   - cpd volume 
c 2 - ephi(k5)   - cpd enthalpy 
c 3 - grunp(k5)  - cpd gruenesien ratio
c 4 - bulkmp(k5) - cpd adiabatic bulk modulus
c 5 - mup(k5)    - cpd shear modulus
c 6 - vel0p(k5)  - cpd sound velocity 
c 7 - velsp(k5)  - cpd s-wave velocity
c 8 - velpp(k5)  - cpd p-wave velocity
c 9 - vp/vs
c10 - rhop(k5)   - cpd density
c11 - vfract(k5) - cpd volume fraction
c12 - cp
c13 - alpha
c14 - beta 


c 1 vsys       - sys molar volume 
c 2 esys       - sys molar enthalpy 
c 3 grun       - sys gruenesien ratio
c 4 bulkm      - sys adiabatic bulk modulus
c 5 mu         - sys shear modulus
c 6 vel0       - sys sound velocity 
c 7 velp       - sys p-wave velocity
c 8 vels       - sys s-wave velocity
c 9 vp/vs      
c 10 rsys      - sys density
c 11 csys      - sys specific heat capacity
c 12 cp
c 13 alpha
c 14 beta 

      implicit real (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h' 

      parameter (l3=l2+2)

      real pgeo(i7), pgeo1(i7)

      common/ cst74 /idbk(k5,k2),ivar(k2),iap(k2),ibulk,iflu(k2),
     *               iflug(k5,k2)
     *      / cst76 /props(i8,k5,k2),vr(l2,k2),vtemp(2,k2)
     *      / cst77 /ivol,jvol(2),jpt(2),jv(l3),jbulk,icp,pmn,dpr
     *      / cst81 /ptot(i8,k2),btot(k2),gtot(k2),fbulk(k5,k2)
     *      / cst82 /ptot1(i8,k2),btot1(k2),gtot1(k2),fbulk1(k5,k2)
     *      / cst317 /pphi(i7,k5),psys(i7),psys1(i7),sphi(i7,k5),
     *                idphi(k5),nphi
     *      / cst319 /iemod(k2),kmod/ phase /ikp(k1)  
c                                      determine if x and/or y are 
c                                      pressure or temperature:
      do j = 1, i7
         psys(j) = 0d0
         pgeo(j) = 0d0
         psys1(j) = 0d0
         pgeo1(j) = 0d0
      end do
c                                       initial true phase counter and props
      nphi = 0
      jtic = 0
      ktic = 0 

      do i = 1, jbulk
         do j = 1, i7
            sphi(j,i) = 0d0
         end do 
      end do 
c                                    first loop gets basic props
c                                    and simple means:
      do i = 1, jbulk

         idphi(i) = 0
c                                    cpd id:
         id = idbk(i,jd)
c                                    bad assemblage, bad exit
         if (id.eq.0) goto 99
c                                    wt for molar wieghted system props
         wt     = props(1,i,jd)
c                                    cpd molar volume
         pphi(1,i) = props(4,i,jd) 
c                                    + dv dp at Tr
     *               +(props(7,i,jd) + props(10,i,jd)*dp/2d0)*dp
c                                    + dv dt at P
     *               +(props(6,i,jd) + props( 9,i,jd)*dp
     *                               + props( 9,i,jd)*dt/2d0)*dt
c                                    cpd heat capacity
         pphi(12,i) = props(5,i,jd) 
c                                    cpd expansivity
         pphi(13,i) = (props(6,i,jd) + props( 9,i,jd)*dp
     *               + (props( 8,i,jd)+props(11,i,jd)/2d0*dt)*dt)
     *                 /pphi(1,i)
c                                    cpd compressibility
         pphi(14,i) = -(props(7,i,jd) + props( 9,i,jd)*dt
     *                               + props(10,i,jd)*dp)/pphi(1,i)
c                                    cpd molar enthalpy, approx
         pphi(2,i) = (props(3,i,jd) 
c                                    + cp dt
     *              + dt * props(5,i,jd) 
c                                    + v dp 
     *              + dp * pphi(1,i))
c                                    system molar volume
         psys(1) = psys(1) + pphi(1,i)*wt
c                                    molar enthalpy
         psys(2) = psys(2) + pphi(2,i)*wt
c                                    molar heat capacity
         psys(12) = psys(12) + pphi(12,i)*wt
c                                    molar expansivity
         psys(13) = psys(13) + pphi(13,i)*wt
c                                    molar compressibility
         psys(14) = psys(14) + pphi(14,i)*wt

         if (iflu(jd).eq.1) then 
c                                    assemblage includes fluid
            if (iflug(i,jd).eq.0) then
c                                    get total without fluid
               psys1(1) = psys(1) + pphi(1,i)*props(1,i,jd)
c                                    molar enthalpy
               psys1(2) = psys1(2) + pphi(2,i)*wt
c                                    molar heat capacity
               psys1(12) = psys1(12) + pphi(12,i)*wt
c                                    molar expansivity
               psys1(13) = psys1(13) + pphi(13,i)*wt
c                                    molar compressibility
               psys1(14) = psys1(14) + pphi(14,i)*wt
            end if 
         end if 
      end do 
c                                    total volume now can be used
c                                    for second loop
      do i = 1, jbulk
c                                    cpd id:
         id = idbk(i,jd)
c                                    volume fraction
         pphi(11,i) = props(1,i,jd)*pphi(1,i)/psys(1)      
c                                    ------------------------------------
c                                    create derivative properties:
c                                    for each compound:     
  
c                                    Thermal Grueneisen Ratio (dimensionless)
         pphi(3,i) = pphi(1,i)*pphi(13,i)/
     *                      pphi(14,i)/(pphi(12,i) - 
     *                      t*pphi(13,i)**2*
     *                       pphi(1,i)/pphi(14,i))
c                                    Adiabatic Bulk Modulus (bar)
         pphi(4,i) = (1d0 + t*pphi(13,i)
     *                       *pphi(3,i))/pphi(14,i)
c                                    Shear Modulus (bar)
         pphi(5,i) = props(12,i,jd) + p*props(13,i,jd) 
     *                              + (t-300d0)*props(14,i,jd)
c                                    density (kg/m3)
         pphi(10,i) = props(2,i,jd)/pphi(1,i)*1e2

         if (pphi(5,i).ne.0d0) then 
c                                    p-wave velocity
            pphi(7,i) = dsqrt((pphi(4,i)+4d0*pphi(5,i)/3d0)*1d5
     *                  /pphi(10,i))/1d3
c                                    s-wave velocity
            pphi(8,i) = dsqrt(pphi(5,i)*1d5/pphi(10,i))/1d3

            pphi(9,i) = pphi(7,i)/pphi(8,i)

         end if 
c                                    sound velocity (km/s)
         if (pphi(4,i).gt.0d0.and.pphi(10,i).gt.0d0) then 
            pphi(6,i) = dsqrt(pphi(4,i)*1d5/pphi(10,i))/1d3
         else 
            pphi(6,i) = 0d0
         end if 


         if (ikp(id).ne.iold.or.ikp(id).eq.0) then 
c                                    a new solution, initialize:
            nphi = nphi + 1
            rcpd  = 0d0
            idphi(nphi) = id
            iold = ikp(id)
            rcpd   = 0d0 
            twt    = 0d0
            vol    = 0d0
            enth   = 0d0
            rho    = 0d0
            cp     = 0d0
            alpha  = 0d0
            beta   = 0d0
            shearm = 0d0
            vmode  = 0d0 
         end if 
c                                   sum properties
         rcpd   = rcpd + 1d0
         wt     = props(1,i,jd)
         twt   = twt + wt 
         vol    = vol + wt*pphi(1,i)
         enth   = enth + wt*pphi(2,i)
         rho    = rho + wt*pphi(10,i)
         cp     = cp  + wt*pphi(12,i)
         alpha  = alpha + wt*pphi(13,i) 
         beta   = beta + wt*pphi(14,i)
         shearm = shearm + wt*pphi(5,i)
         if (shearm.ne.0d0.and.iflug(i,jd).eq.0) jtic = jtic + 1
         if (iflu(jd).eq.1) then 
            if (iflug(i,jd).eq.1) ktic = ktic + 1
         end if 
         vmode  = vmode + pphi(11,i)

         if (i.eq.jbulk.or.ikp(id).eq.0) then 
            iend = 1
         else if (ikp(idbk(i+1,jd)).ne.iold) then 
            iend = 1
         else 
            iend = 0 
         end if 

         if (iend.eq.1) then 
c                                  compute phase props:
            sphi(1,nphi)  = vol/twt
            sphi(2,nphi)  = enth/twt
            sphi(5,nphi)  = shearm/twt
            sphi(10,nphi) = rho/twt
            sphi(11,nphi) = vol/psys(1) 
            sphi(12,nphi) = cp/twt
            sphi(13,nphi) = alpha/twt
            sphi(14,nphi) = beta/twt
            sphi(15,nphi) = twt/rcpd
            sphi(16,nphi) = vmode
c                                    Thermal Grueneisen Ratio (dimensionless)
            sphi(3,nphi) = sphi(1,nphi)*sphi(13,nphi)/sphi(14,nphi)/
     *                  (sphi(12,nphi) - t*sphi(13,nphi)**2*
     *                  sphi(1,nphi)/sphi(14,nphi))
c                                    Adiabatic Bulk Modulus (bar)
            sphi(4,nphi) = (1d0 + t*sphi(13,nphi)*sphi(3,nphi))
     *                       /sphi(14,nphi)
c                  
           if (sphi(5,nphi).ne.0d0.and.twt.gt.0d0) then 
c                                    p-wave velocity
              sphi(7,nphi) = dsqrt((sphi(4,nphi)+4d0*sphi(5,nphi)/3d0)
     *                            *1d5/sphi(10,nphi))/1d3
c                                    s-wave velocity
              sphi(8,nphi) = dsqrt(sphi(5,nphi)*1d5/sphi(10,nphi))/1d3

              sphi(9,nphi) = sphi(7,nphi)/sphi(8,nphi)

            end if 
c                                    sound velocity (km/s)
            if (sphi(4,nphi).gt.0d0.and.twt.gt.0d0) then 
               sphi(6,nphi) = dsqrt(sphi(4,nphi)*1d5/sphi(10,nphi))/1d3
            else 
               sphi(6,nphi) = 0d0
            end if 
c                                    compute system sums:
c                                    These depend on weighting scheme,
c                                    here we use Voigt-Reuss-Hill.
c                                    Aggregate Gruneisen Ratio
            psys(3) = psys(3) + sphi(11,nphi)*sphi(3,nphi)/2d0
            pgeo(3) = pgeo(3) + sphi(11,nphi)/sphi(3,nphi)
c                                    Aggregate Bulk Modulus                                
            psys(4) = psys(4) + sphi(11,nphi)*sphi(4,nphi)/2d0
            pgeo(4) = pgeo(4) + sphi(11,nphi)/sphi(4,nphi)
c                                    Aggregate Sound
            psys(6) = psys(6) + sphi(11,nphi)*sphi(6,nphi)/2d0
            pgeo(6) = pgeo(6) + sphi(11,nphi)/sphi(6,nphi)

            if (iemod(jd).eq.jbulk) then 
c                                    Aggregate Shear Modulus, only
c                                    if shear mod is available for all
c                                    cpds.
               psys(5) = psys(5) + sphi(11,nphi)*sphi(5,nphi)/2d0
               pgeo(5) = pgeo(5) + sphi(11,nphi)/sphi(5,nphi)
c                                    Aggregate Vp                                
               psys(7) = psys(7) + sphi(11,nphi)*sphi(7,nphi)/2d0
               pgeo(7) = pgeo(7) + sphi(11,nphi)/sphi(7,nphi)
c                                    Aggregate Vp                                
               psys(8) = psys(8) + sphi(11,nphi)*sphi(8,nphi)/2d0
               pgeo(8) = pgeo(8) + sphi(11,nphi)/sphi(8,nphi)
            end if

            if (iflu(jd).eq.1) then 
c                                    compute totals without fluid:
               if (iflug(i,jd).eq.0) then 
c                                    renormalized fraction:
                  frac = sphi(11,nphi)*psys(1)/psys1(1)
c                                    Aggregate Gruneisen Ratio
                  psys1(3) = psys1(3) + frac*sphi(3,nphi)/2d0
                  pgeo1(3) = pgeo1(3) + frac/sphi(3,nphi)
c                                     Aggregate Bulk Modulus                                
                  psys1(4) = psys1(4) + frac*sphi(4,nphi)/2d0
                  pgeo1(4) = pgeo1(4) + frac/sphi(4,nphi)
c                                     Aggregate Sound vel                                
                  psys1(6) = psys1(6) + frac*sphi(6,nphi)/2d0
                  pgeo1(6) = pgeo1(6) + frac/sphi(6,nphi)

                  if (sphi(5,nphi).ne.0) then 
c                                    Aggregate Shear Modulus, only
c                                    if shear mod is available for all
c                                    cpds.
                      psys1(5) = psys1(5) + frac*sphi(5,nphi)/2d0
                      pgeo1(5) = pgeo1(5) + frac/sphi(5,nphi)
c                                    Vp
                      psys1(7) = psys1(7) + frac*sphi(7,nphi)/2d0
                      pgeo1(7) = pgeo1(7) + frac/sphi(7,nphi)
c                                    Vs
                      psys1(8) = psys1(8) + frac*sphi(8,nphi)/2d0
                      pgeo1(8) = pgeo1(8) + frac/sphi(8,nphi)
                 end if
              end if 
            end if 
         end if 

      end do 

      psys(3) = psys(3) + 0.5d0/pgeo(3)
      psys(4) = psys(4) + 0.5d0/pgeo(4)
c                                    density
      psys(10) = ptot(2,jd)/psys(1)*1e2
c                                    sound velocity
      if (psys(4).gt.0d0.and.psys(10).gt.0d0) then 
         psys(6) = dsqrt(psys(4)*1d5/psys(10))/1d3
      else 
         psys(6) = 0d0
      end if 

      if (iemod(jd).eq.jbulk) then 
c                                    shear modulus
         psys(5) = psys(5) + 0.5d0/pgeo(5)
c                                    p-wave velocity
         psys(7) = psys(7) + 0.5d0/pgeo(7)
c                                    s-wave velocity
         psys(8) = psys(8) + 0.5d0/pgeo(8)

         psys(9) = psys(7)/psys(8)
      else 
         psys(5) = 0d0
         psys(7) = 0d0
         psys(8) = 0d0
      end if 

      if (iflu(jd).eq.1) then 
c                                    assemblage contains fluid, compute
c                                    fluid absent properties:
c                                    molar volume
         psys1(1) = ptot1(4,jd)*(1e0 + dt * ptot1(6,jd) - 
     *                                 dp * ptot1(7,jd))
c
         psys1(2) = (ptot1(3,jd) + dt * ptot1(5,jd) + 
     *                             dp * psys1(1))/psys1(1)*1e5  
         psys1(3) = psys1(3) + 0.5d0/pgeo1(3)
         psys1(4) = psys1(4) + 0.5d0/pgeo1(4) 
c                                    sound velocity
         psys1(6) = psys1(6) + 0.5d0/pgeo1(6)
         psys1(10) = ptot1(2,jd)/psys1(1)*1e2

         if (jtic.eq.jbulk-ktic) then 
c                                    shear modulus
            psys1(5) = psys1(5) + 0.5d0/pgeo1(5)
c                                    p-wave velocity
            psys1(7) = psys1(7) + 0.5d0/pgeo1(7)
c                                    s-wave velocity
            psys1(8) = psys1(8) + 0.5d0/pgeo1(8)

            psys1(9) = psys1(7)/psys1(8)
         else 
            psys1(5) = 0d0
            psys1(7) = 0d0
            psys1(8) = 0d0
         end if 
         psys1(6) = dsqrt(psys1(4)*1d5/psys1(10))/1d3

      end if 

99    end 
