!!!PROGRAM POLYGON   (EVERYTHING SQUEEZED IN ONE FILE)
!besides the main for the program polygon this file contains the modules: 
!geom_lib: a set of routines to do geometrical tasks (find intersections,centers,...)
!input_output: routines for reading the various files (pl,bpl,..)+ interactive reading
!              + rountines of preparationfor the output
!build_polygon: core of the program, all routines for binding together the segments into polygons
!the program also requires the routines contained in the file jamie2.f
!********************************************************************************
module geom_lib
!this module contains a set of subrotuines that perform geometrical tasks
!e.g. find the intersection between two lines,...
contains
!--------------------------------------------------------------------------------------
subroutine check_crossing(p1,p2,q1,q2,r1,r2,s1,s2,errort,errorp,crossing)
!this subroutine checks if segments are crossing each other inside the segment
!                 s(s1,s2)  *          * Q(q1,q2)
!                             \      /
!                               \  /     y=((q2-p2)/(q1-p1))*(x-p1)+p2
!y=((r2-s2)/(r1-s1))*(x-s1)+s2   / X (x,y)  
!                              /    \
!                            /        \
!                           *           *
!                      P(p1,p2)     R(r1,r2)
implicit none
real, intent (in)    :: p1,p2,q1,q2,r1,r2,s1,s2 
real, intent (in)    :: errort,errorp   !tollerancy for the error in interpol.
real                 :: x,y             !coor of intersection
logical,intent (out) :: crossing

crossing=.false.
x=(p1*q2*r1-p1*q2*s1+s2*q1*r1-s2*p1*r1-p2*q1*r1+p2*q1*s1-s1*r2*q1+s1*r2*p1)&
 & /(-r2*q1+r2*p1+s2*q1-s2*p1+q2*r1-q2*s1-p2*r1+p2*s1)  !x and y are the coordinate of the inteception
y=(q2-p2)/(q1-p1)*((p1*q2*r1-p1*q2*s1+s2*q1*r1-s2*p1*r1-p2*q1*r1+p2*q1*s1&
  &-s1*r2*q1+s1*r2*p1)/(-r2*q1+r2*p1+s2*q1-s2*p1+q2*r1-q2*s1-p2*r1+p2*s1)-p1)+p2
if (abs((q1-x))<errort.or.abs((p1-x))<errort.and.&     !if the crossing is at the boundary it doesn't matter (need an error because calculation is not precise
    & abs((q2-y))<errorp.or.abs((p2-y))<errorp) return !need to check only one because if on extreme all the same 
if ((p1-x)*(q1-x)<0.and.((p2-y)*(q2-y))<0) then        !check if the interseption is inside both segments
 if ((r1-x)*(s1-x)<0.and.((r2-y)*(s2-y))<0) then
    crossing=.true.
  endif  
endif
end subroutine check_crossing
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine baricenter (x,y,p1,p2,r1,r2,a1,a2)
!this sub. finds the baricenter of tha triangle
implicit none
real,intent (in) :: p1,p2,r1,r2,a1,a2  !vertex of trinagles
real,intent (out):: x,y                !baricenter of triangle
real     :: q1,q2,s1,s2        !median of side (facing vertex)

q1=(r1+a1)/2; q2=(r2+a2)/2
s1=(p1+a1)/2; s2=(p2+a2)/2

x=(p1*q2*r1-p1*q2*s1+s2*q1*r1-s2*p1*r1-p2*q1*r1+p2*q1*s1-s1*r2*q1+s1*r2*p1)&
  & /(-r2*q1+r2*p1+s2*q1-s2*p1+q2*r1-q2*s1-p2*r1+p2*s1)  !x and y are the coordinate of the inteception
y=(q2-p2)/(q1-p1)*((p1*q2*r1-p1*q2*s1+s2*q1*r1-s2*p1*r1-p2*q1*r1+p2*q1*s1&
   &-s1*r2*q1+s1*r2*p1)/(-r2*q1+r2*p1+s2*q1-s2*p1+q2*r1-q2*s1-p2*r1+p2*s1)-p1)+p2

endsubroutine baricenter
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine center(coords,x,y)
!--------------------------------------------------------------------------------
!this subroutine finds the geometrical center of a polygon using all the coord
!of the polygon. 
!--------------------------------------------------------------------------------
implicit none
   real,intent(in)  :: coords(:)
   real,intent(out) :: x,y
   integer          :: nocoor,j
!---------------------------------------------------------------------------------
nocoor=ubound(coords,1)
x = 0 ; y = 0
do j=1,nocoor,2
    x=x+coords(j);
    y=y+coords(j+1)
enddo 
x=x/(nocoor/2)
y=y/(nocoor/2)
endsubroutine center
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine check_inside (x,y,line_x,line_y,inside)
!this subrotuine checks if point with given coordinates lies inside or outside
!the area defiend by a set of coordinates
!assuming that the first and last coordinates of the polygon are the same
implicit none
real, intent(in)     :: x,y                            !coordinate of the point to check
real, intent (in)    :: line_x(:),line_y(:)            !cordinate of line defining area
logical,intent (out) :: inside                         !true if point is inside area
integer              :: nopoints                       !no of set of coord defining area
integer              :: i
!--------------------------------------------------------------------------------------------
nopoints        = ubound(line_x,1)  !maybe nopoint=size(line_x,1)
inside         = .true.

do i=1,nopoints-1
   if ( ((y .le. line_y(i)) .and. (y .gt. line_y(i+1))) .or. ((y .gt. line_y(i)) .and. (y .le. line_y(i+1))) ) then 


 if (((((line_y(i+1)-line_y(i))*(x-line_x(i))-(y-line_y(i))*(line_x(i+1)-line_x(i)))*(line_y(i+1)-line_y(i)) < 0))) then
      inside = .not. inside
   endif

   endif
enddo

inside = .not. inside
endsubroutine check_inside


endmodule geom_lib
!************************************************************************************
!************************************************************************************
!************************************************************************************
module input_output

use geom_lib

integer               :: maxncoor=200      !per segment
integer               :: maxseg=20000
integer               :: noass=8000        !max no of assemglages=~no polygons
integer               :: maxseg_per_pol=15 !max no of segments forming a polygon
integer,parameter     :: maxcoorpol=500
integer               :: nopol

integer,allocatable   :: seg_to_ass(:,:)    !array storing all the segments bounding each aseemblage
integer,allocatable   :: countseg_to_ass(:) !no of segments with given assemblage
integer               :: maxnoass           !no of assembl. present (updated as reading)
real                  :: maxt, mint, maxp, minp    !axes
real, allocatable     :: t(:,:), p(:,:) 
integer,allocatable   :: numcoor(:)
integer,allocatable   :: ass_to_pol(:)
integer               :: countpol
integer,allocatable   :: countcoorpol(:)   !countcoorpol(nopol)
real,allocatable      :: polygon(:,:)      !polygon(maxcoorpol,nopol)
integer,allocatable   :: fin(:)            !fin(nopol): flag 1 is poly is fiished 0 if aborted

contains
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine read_input                                                      
!read input from file and initialize output
implicit none

integer,allocatable   :: n(:)          !dummy array for no of coord in a segment  they will be transfered to 
real*8, allocatable   :: c(:,:)        !coordinates of segment (as read)          optimized size array
character(100)        :: fnamein       !name of file from which input is read
character(100)        :: nname   
integer               :: idummy, idummy1
real                  :: rdummy(4)
character(8)          :: yon
integer               :: i, k, j
logical               :: skip
integer,allocatable   :: assemb(:,:)    !array storing all the segments bounding each aseemblage
!jamies
integer        :: n4,ifont,icopt,jwidth,iop0
common / basic /n4,ifont,icopt,jwidth,iop0
!--------------------------------------------------------------------------
write(*,*) 'Modify memory allocation (y/n)?'
read (*,'(a)') yon 
if (yon=='y') then
!    call change_allocation(noass,'noass')
    call change_allocation(maxseg,'maxseg')
    call change_allocation(maxncoor,'maxncoor')
    call change_allocation(maxseg_per_pol,'maxseg_per_pol')
endif

allocate (c(maxncoor,maxseg),n(maxseg),assemb(2,maxseg))  !initialization

c=0. ;   n=0           

n4 = 1

call plinp (fnamein,mint,maxt,minp,maxp) ! opens unit 1 
!                                        jcopt is the computational option
!                                        flag written by routine header in 
!                                        vertex, it is set to 6 for polygonalized
!                                        pseudosections. write warning if user
!                                        is trying to anlayze the wrong kind of 
!                                        file:
if (icopt.ne.6) then
   if (icopt.eq.5) then
      write (*,1000) 
   else 
      write (*,1010)
   end if 
   stop 
end if 


1000 format (/,'Output for the computational options you selected is analyzed',/,&
               ' with WERAMI (i.e., POLYGON is not necessary).',/)
1010 format (/,'The plot file you have specified does not correspond to a',/,&
               'polygonalized phase diagram section.',/,&
               'Try plotting the file with PSVDRAW.',/)

call bplinp (fnamein,noass)  !subrotuine written by Jamie to read info about density, water%,...
noass = noass + 100
nopol = noass 
allocate (seg_to_ass(maxseg_per_pol,noass),countseg_to_ass(noass))

countseg_to_ass=0;seg_to_ass=0
maxnoass=0 



!call bplinp (fnamein,nopol)  !subrotuine written by Jamie to read info about density, water%,...


nname = fnamein
call inblnk (nname,'p')
open (2,file=nname)

k=1                                      !reading input for univariant fields
univariant: do                           !open end do because no of segments is unknown yet
   if (k.gt.maxseg) call warn_allocation('maxseg')                              
   read(1,*) n(k),idummy,idummy,idummy,(idummy1,i=1,idummy),assemb(1,k),assemb(2,k)         
   if (n(k).lt.2) exit univariant        !the end of the univariant data is indicated in the inout file by a series of 1s                                
   read(1,*) (rdummy(1), i=1,idummy)                                    
   read(1,*) (c(i,k),i=1,n(k))
        !katja changed 14.05.99 this line:
        !if (assemb(2,k).eq.0) then                      ! improperly traced segment, fix this.
        !with this: 
    if (assemb(2,k).eq.assemb(1,k)) then            ! improperly traced segment, fix this.
        !because input has changed. (there are no 0 anymore but twice the same assemblage)
      write (*,*) ' Bad segment for eq ',n(k),' 1st assemblage = ',assemb(1,k)
      cycle
    end if 
 
   if (assemb(1,k).gt.maxnoass) maxnoass=assemb(1,k)  !update no of assembl. present
   if (assemb(2,k).gt.maxnoass) maxnoass=assemb(2,k)
   if (maxnoass.gt.noass) call warn_allocation('noass') 
   if (n(k).gt.maxncoor) call warn_allocation('maxncoor') 

   call check_new (k,skip,assemb)                      !checks if not duplicate

   if (.not.skip) then                   !in case of duplicate does not record data 
     countseg_to_ass(assemb(1,k))=countseg_to_ass(assemb(1,k))+1  
     if (countseg_to_ass(assemb(1,k)).gt.maxseg_per_pol) call warn_allocation('maxseg_per_pol')
     countseg_to_ass(assemb(2,k))=countseg_to_ass(assemb(2,k))+1   
     if (countseg_to_ass(assemb(2,k)).gt.maxseg_per_pol) call warn_allocation('maxseg_per_pol')

     seg_to_ass(countseg_to_ass(assemb(1,k)),assemb(1,k))=k
     seg_to_ass(countseg_to_ass(assemb(2,k)),assemb(2,k))=k
   endif
   k=k+1
enddo univariant

maxncoor=maxval(n)/2 ;   maxseg=k-1              !...calculate real max size of arrays...
allocate (numcoor(maxseg))                       !and rewrite the input data in optimized sized array.
numcoor=pack(n,n.gt.0);   numcoor=numcoor/2      !numcoor/2 because in input p,t are in the same array
deallocate (n)
allocate (t(maxncoor,maxseg),p(maxncoor,maxseg))
p=0;t=0
do i=1,maxseg                                                           !create optimized coor array 
  if (c(1,i).le.c((2*numcoor(i))-1,i)) then                            !arranging lines so that written with temp
    do j=1,numcoor(i)                                                  !generally increasing
      t(j,i)=c(2*j-1,i) ;  p(j,i)=c(2*j,i)
    enddo
  else
     do j=1,numcoor(i)
       t(j,i)=c(2*numcoor(i)-2*j+1,i);  p(j,i)=c(2*numcoor(i)-2*j+2,i)
     enddo
  endif
enddo
deallocate (c)
endsubroutine read_input
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine check_new (k,skip,assemb)
!this subroutine checks if a segment is new or if it is a duplicate, i.e.
!asegment with the same two aseemblages exists already
implicit none
integer, intent (in) :: k,assemb(:,:)
logical, intent (out):: skip
integer   :: n
skip=.false.
do n=1,k-1
   if (assemb(1,k).eq.assemb(1,n).and.assemb(2,k).eq.assemb(2,n)) skip=.true.
   if (assemb(1,k).eq.assemb(2,n).and.assemb(2,k).eq.assemb(1,n)) skip=.true.
enddo
end subroutine check_new
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine output
  implicit none
  integer  :: i,j
  real     :: x,y
  logical  :: inside


write (*,*) 'Preparing output'

write(2,*) countpol, maxval(countcoorpol)
do i=1,countpol
  call center (polygon(1:countcoorpol(i),i),x,y)
  call check_inside(x, y, polygon(1:countcoorpol(i):2,i) , polygon(2:countcoorpol(i):2,i) ,inside)
!                  open (25,file='test')
!                  do j=1,countcoorpol(i),2
!                  write (25,*) polygon(j,i), polygon(j+1,i)
!                  enddo
!                  write (25,*) x,y
!                  close(25)
!  if (.not.inside.and.fin(i).ne.0.and.countcoorpol(i).gt.6)&
!                       call baricenter (x,y,polygon(1,i),polygon(2,i),polygon(3,i),polygon(4,i)&   !finds a point inside each polygon placed              
!                      &   ,polygon(countcoorpol(i)-3,i),polygon(countcoorpol(i)-2,i))             !just inside the first corner. 


  if (.not.inside) then 
   x = polygon(1,i)
   y = polygon(2,i)
  end if 

  write(2,*) countcoorpol(i),ass_to_pol(i),fin(i),x,y;
  write(2,*) (polygon(j,i),j=1,countcoorpol(i))

enddo

close (2); 

endsubroutine output
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine change_allocation(param,cparam)
   implicit none
   integer :: param
   character(*) :: cparam
   character(1) :: yon
write (*,*) 'Change allocation for ',cparam,' (',param,') y/n?'
read(*,'(a1)') yon
if (yon=='y') then
	  write (*,*) 'enter new value:'
	  read  (*,*) param
endif
endsubroutine change_allocation
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine warn_allocation(cparam)
   implicit none
   character(*) :: cparam
write(*,*) 'Insufficient memory'
write(*,*) 'Increase allocation for ',cparam
write(*,*) 'Press return to stop the program'
pause
stop
endsubroutine warn_allocation
endmodule input_output
!************************************************************************************
!************************************************************************************
!************************************************************************************
module build_poly

use input_output
use geom_lib

integer,allocatable   :: flag(:,:) !(0 if segment has not been used yet 1 if used)
integer               :: found     !(0 if neighbour not found 1 if found start 2 if found end)
real                  :: start(2),actual(2)
contains
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine build
implicit none
integer  :: i,n
!-------------------------------------------------------------------
write (*,*) 'Constructing polygons'
allocate(flag(maxseg,noass));flag=0
allocate(countcoorpol(nopol),polygon(maxcoorpol,nopol),fin(nopol))
allocate (ass_to_pol(nopol))
countpol=0 ; countcoorpol=0; fin=0;

do i=1,maxnoass                    !i=id of assemblage -->polygon
  if (countseg_to_ass(i).lt.1) cycle 
  countpol=countpol+1 
!                           if (countpol.gt.nopol) call warn_allocation(maxpol)
  ass_to_pol(countpol)=i
  fin(countpol)=1                  !will be set back to 0 if aborted

  start(1)=t(1,seg_to_ass(1,i))
  start(2)=p(1,seg_to_ass(1,i))
  found=1                  !necessery for the direction on update
  call update (seg_to_ass(1,i)) ;  flag(1,i)=1
  if (countseg_to_ass(i)==1) then !case in which polygon is only bound by one non-boundary segment
     call onesegpolygon
  else
    do while (actual(1).ne.start(1).or.actual(2).ne.start(2))  !i.e. until poly is closed 
 	   call findnext (i,n)
       if (found.ne.0) then
	      call update(seg_to_ass(n,i)); flag(n,i)=1  
	   else
	     if (actual(1).eq.mint.or.actual(1).eq.maxt.or.actual(2).eq.minp.or.actual(2).eq.maxp) then
  !segment on boundary
	       if (countseg_to_ass(i).gt.1) call search_boundary (i,n)    !first look along same boundary
	       if (found.ne.0) then
	           call update(seg_to_ass(n,i)); flag(n,i)=1
	       else 
	           call corner  (i,n)         !then look along other boundaries
		       if (found==0) call finished (i,n) !the polygon might have been finished
	       endif 
         endif
	   endif
	   if (found.eq.0) then              !polygon has to be aborted
         write(*,*) 'Polygon bounding assemblage ',i,' is incomplete'
	     fin(countpol)=0
	     exit
       endif
    enddo
  endif
enddo
endsubroutine build
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
subroutine update (k)  !k=seg_to_ass(k,i)
!this sub. updates the coordinate of the polygon including the last segment and returns the
!actual, i.e. the last point of the polygon that has to be continued
implicit none
integer,intent (in)  :: k
integer  :: a,b,c,j
if (found.eq.1) then
  a=1;  b=numcoor(k) ;c=1
else
  a=numcoor(k); b=1 ;c=-1
endif
do j=a,b,c
    countcoorpol(countpol)=countcoorpol(countpol)+2
    if (countcoorpol(countpol).gt.maxcoorpol) call warn_allocation('maxcoorpol')
    polygon(countcoorpol(countpol)-1,countpol) =t(j,k)
	polygon(countcoorpol(countpol),countpol) =p(j,k)
enddo
  actual(1)=t(b,k);  actual(2)=p(b,k)
endsubroutine update
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
subroutine findnext(i,n)
implicit none
!this subrotuine looks in the array of segments with the same assemblage for the 
!segments that starts (found=1) or ends (found=2) at actual
integer,intent (in)  :: i 
integer,intent (out) :: n
found=0
do n=2,countseg_to_ass(i)  !amongst other segments with same assemblage find nighbour
   if (flag(n,i).gt.0) cycle
   if (t(1,seg_to_ass(n,i)).eq.actual(1)) then
      if (p(1,seg_to_ass(n,i)).eq.actual(2)) then 
         found=1 ; exit
      endif
   else if (t(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.actual(1)) then
      if (p(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.actual(2)) then 
	      found=2; exit
	  endif
   endif
enddo
endsubroutine findnext
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
subroutine search_boundary (i,n)
implicit none 
integer, intent (out)  :: n
integer,intent  (in)   :: i
if (actual(1).eq.mint.or.actual(1).eq.maxt) then
   do n=2,countseg_to_ass(i)  !amongst other segments with same assemblage find nighbour
      if (flag(n,i).gt.0) cycle
      if (t(1,seg_to_ass(n,i)).eq.actual(1)) then
         found=1;exit
      else if (t(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.actual(1)) then
         found=2 ; exit
      endif
   enddo
elseif (actual(2).eq.minp.or.actual(2).eq.maxp) then
   do n=2,countseg_to_ass(i)  !amongst other segments with same assemblage find nighbour
      if (flag(n,i).gt.0) cycle
	  if (p(1,seg_to_ass(n,i)).eq.actual(2)) then
         found=1;exit
      else if (p(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.actual(2)) then
         found=2 ; exit
      endif
   enddo
endif
end subroutine search_boundary
!-------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------
subroutine corner (i,n)
!this subroutine handels the case in which a segment finishes on bundary and next 
!one starts around the corner
implicit none
integer, intent (out) :: n
integer, intent (in)  :: i
do n=2,countseg_to_ass(i)  !amongst other segments with same assemblage find nighbour
  if (flag(n,i).gt.0) cycle
  if (t(1,seg_to_ass(n,i)).eq.mint) then
    found=1
    if (actual(1).ne.maxt) then  !cioe c'e solo un corner
      countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =mint
	  polygon(countcoorpol(countpol),countpol) =actual(2)
	else  
	  call two_corners(maxt,minp,mint,maxp,n,i)
	endif
    call update(seg_to_ass(n,i));flag(n,i)=1
	exit
  elseif (t(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.mint) then
    found=2
	if (actual(1).ne.maxt) then
      countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =mint
	  polygon(countcoorpol(countpol),countpol) =actual(2)
    else  
	  call two_corners(maxt,minp,mint,maxp,n,i)
	endif
    call update(seg_to_ass(n,i));flag(n,i)=1
	exit
  elseif (t(1,seg_to_ass(n,i)).eq.maxt) then
    found=1
    if (actual(1).ne.mint) then
      countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =maxt
	  polygon(countcoorpol(countpol),countpol) =actual(2)
    else  
	  call two_corners(mint,minp,maxt,maxp,n,i)
	endif
	call update(seg_to_ass(n,i));flag(n,i)=1
	exit
  elseif (t(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.maxt) then
    found=2
	if (actual(1).ne.mint) then
      countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =maxt
	  polygon(countcoorpol(countpol),countpol) =actual(2)
    else  
	  call two_corners(mint,minp,maxt,maxp,n,i)
	endif
    call update(seg_to_ass(n,i));flag(n,i)=1
	exit
  elseif (p(1,seg_to_ass(n,i)).eq.minp) then
    found=1
    if (actual(2).ne.maxp) then
	  countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =actual(1)
	  polygon(countcoorpol(countpol),countpol) =minp
	else
	   call two_corners(mint,maxp,maxt,minp,n,i)
    endif
    call update(seg_to_ass(n,i)); flag(n,i)=1
	exit
  elseif (p(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.minp) then
    found=2
	if (actual(2).ne.maxp) then
	  countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =actual(1)
	  polygon(countcoorpol(countpol),countpol) =minp
	else
	   call two_corners(mint,maxp,maxt,minp,n,i)
    endif
    call update(seg_to_ass(n,i));flag(n,i)=1
    exit
  elseif (p(1,seg_to_ass(n,i)).eq.maxp) then
    found=1
    if (actual(2).ne.minp) then
	  countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =actual(1)
	  polygon(countcoorpol(countpol),countpol) =maxp
	else
	   call two_corners(mint,minp,maxt,maxp,n,i)
    endif
	call update(seg_to_ass(n,i));flag(n,i)=1
	exit
  elseif (p(numcoor(seg_to_ass(n,i)),seg_to_ass(n,i)).eq.maxp) then
    found=2
    if (actual(2).ne.minp) then
	  countcoorpol(countpol)=countcoorpol(countpol)+2
      polygon(countcoorpol(countpol)-1,countpol) =actual(1)
	  polygon(countcoorpol(countpol),countpol) =maxp
	else
	   call two_corners(mint,minp,maxt,maxp,n,i)
    endif
	call update(seg_to_ass(n,i));
	flag(n,i)=1
	exit
  endif
enddo

end subroutine corner
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine finished (i,n)
!this subroutine takes into accoutn the fact that the polygon migth have
!been finished, i.e. had only to go along boundary (or corner) to find 
!starting point. The segment containing the starting point however could
!not be found by the previous subroutines because it was flagged 1, i.e. 
!already used
!Do not try avoiding this problem by supressing the flag test in the previous
!subrotines because it is necesssary to find the right continuation when polygon
!goes round a corner otherwise it might try to lik to already used segments
implicit none
integer, intent(in) :: i
!integer, intent(in/out) :: n
integer :: n

if (actual(1).eq.mint.or.actual(1).eq.maxt) then
   if (start(1).ne.actual(1)) then  !polygon has to go around the corner
      if (start(1).eq.mint.and.actual(1).eq.maxt.or.&
	 &   start(1).eq.maxt.and.actual(1).eq.mint) then !two corners
         call onesegpolygon
!         call two_corners(actual(1),minp,start(1),maxp,n,i)
!   	  call update(seg_to_ass(n,i));flag(n,i)=1
	  else !just one corner
		 countcoorpol(countpol)=countcoorpol(countpol)+2
         polygon(countcoorpol(countpol)-1,countpol) =actual(1)
	     polygon(countcoorpol(countpol),countpol) =start(2)
	  endif
   endif
   found=1         !has to be not 0 otherwise program thinks it has been aborted 
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =start(1)
   polygon(countcoorpol(countpol),countpol) =start(2)
   actual=start
elseif (actual(2).eq.minp.or.actual(2).eq.maxp) then
   if (start(2).ne.actual(2)) then  !polygon has to go around the corner
      if (start(2).eq.minp.and.actual(2).eq.maxp.or.&
     &   start(2).eq.maxp.and.actual(2).eq.minp) then !two corners
         call onesegpolygon
!	     call two_corners(mint,actual(2),maxt,start(2),n,i)
!	     call update(seg_to_ass(n,i));flag(n,i)=1
	  else
         countcoorpol(countpol)=countcoorpol(countpol)+2
         polygon(countcoorpol(countpol)-1,countpol) =start(1)
	     polygon(countcoorpol(countpol),countpol) =actual(2)
      endif
   endif
   found=1  !has to be not 0 otherwise program thinks it has been aborted 
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =start(1)
   polygon(countcoorpol(countpol),countpol) =start(2)
   actual=start
endif

endsubroutine finished
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine two_corners (a,b,c,d,n,i)
!this sbub. creates 2 points (on the corners) for polygones whcoh have to go round two corners
!the main difficulty is that there are two possible way (i.e. two possible corners)
!to determine which way is rigth a segment gets created between extremity of next (starting)
!segment and 1st point to be created. IF that segment intersects the boundary of the polygon 
!it is the wrong way
!the variables a,b,c,d vary according to the boundary in which segm, stop and start.
!case1) last ends on maxt and starts on mint:  (maxt,minp,mint,maxp)
!case2) last ends on mint and starts on maxt:  (mint,minp,maxt,maxp)
!case3)  stops on maxp and start on minp  :(mint,maxp,maxt,minp)
!case4) (mint,minp,maxt,maxp)
implicit none
!integer :: m,n,i 
integer :: m 
integer, intent (in) :: n,i
real :: a,b,c,d
!integer :: m,n,i 
logical  :: crossing

do m=1,countcoorpol(countpol),2 
   call check_crossing (a,b,t(1,seg_to_ass(n,i)),p(1,seg_to_ass(n,i)),& !first assume that point created is at minp
  &              polygon(m,countpol),polygon(m+1,countpol),polygon(m+2,countpol),polygon(m+3,countpol),&
  &             (maxt-mint)/1e4,(maxp-minp)/1e4,crossing)
enddo 
if (crossing) then !minp was wrong has to be maxp
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =a
   polygon(countcoorpol(countpol),countpol) =d
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =c
   polygon(countcoorpol(countpol),countpol) =d
else
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =a
   polygon(countcoorpol(countpol),countpol) =b
   countcoorpol(countpol)=countcoorpol(countpol)+2
   polygon(countcoorpol(countpol)-1,countpol) =c
   polygon(countcoorpol(countpol),countpol) =b
endif 
end subroutine two_corners
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
subroutine onesegpolygon
!this subrotuine determines the direction in which a polygon has to be finished in the
!case of a polygon mode of only one non-boundary segment
implicit none
integer :: bb,cc
!first determine on which axis if finishes (can only be maxt as t has to increase).

if (actual(1).eq.maxt) then
   seg_loop1 : do bb=1,maxseg
      if (maxseg==1) stop 'calculations with only one segment cannot be treated'
	     if (t(1,bb).eq.maxt.and.p(1,bb).lt.actual(2).or.&
		    t(numcoor(bb),bb).eq.maxt.and.p(numcoor(bb),bb).lt.actual(2)) then !has to continue on maxp
             do cc=1,maxseg    !check that other side is empty of segments
				if (t(1,cc).eq.maxt.and.p(1,cc).gt.actual(2).or.& 
		           t(numcoor(cc),cc).eq.maxt.and.p(numcoor(cc),cc).gt.actual(2)) then			
                   found =0   !abort polygon
				   return
         		else
				   found=1			  
			    endif
			 enddo
			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=maxt  
			polygon(countcoorpol(countpol),countpol)=maxp
			if (start(2).eq.maxp) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=mint  
			   polygon(countcoorpol(countpol),countpol)=maxp
			   if (start(1).eq.mint) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=mint  
			     polygon(countcoorpol(countpol),countpol)=minp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
			 endif
			 exit seg_loop1          
         elseif (t(1,bb).eq.maxt.and.p(1,bb).gt.actual(2).or.&
		 &       t(numcoor(bb),bb).eq.maxt.and.p(numcoor(bb),bb).gt.actual(2)  ) then !has to continue on pmin
            do cc=1,maxseg    !check that other side is empty of segments
				if (t(1,cc).eq.maxt.and.p(1,cc).lt.actual(2).or.& 
		           t(numcoor(cc),cc).eq.maxt.and.p(numcoor(cc),cc).lt.actual(2)) then			
                   found =0   !abort polygon
				   return
         		else
				   found=1			  
			    endif
            enddo
			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=maxt  
			polygon(countcoorpol(countpol),countpol)=minp
			if (start(2).eq.minp) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=mint  
			   polygon(countcoorpol(countpol),countpol)=minp
			   if (start(1).eq.mint) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=mint  
			     polygon(countcoorpol(countpol),countpol)=maxp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
      		 endif
			 exit seg_loop1  		 
		 endif  !(se == significa che e lo stesso segmento)
   enddo seg_loop1 
elseif (actual(2).eq.minp) then
   seg_loop2: do bb=1,maxseg
      if (maxseg==1) stop 'calculations with only one segment cannot be treated'
	     if (p(1,bb).eq.minp.and.t(1,bb).lt.actual(1).or.&
		 &  p(numcoor(bb),bb).eq.minp.and.t(numcoor(bb),bb).lt.actual(1) ) then !has to continue on maxt
           do cc=1,maxseg    !check that other side is empty of segments
		      if (p(1,cc).eq.minp.and.t(1,cc).gt.actual(1).or.&
		 &  p(numcoor(cc),cc).eq.minp.and.t(numcoor(cc),cc).gt.actual(1) ) then			
                   found =0   !abort polygon
				   return
         	  else
				   found=1			  
			  endif
            enddo  
  			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=maxt  
			polygon(countcoorpol(countpol),countpol)=minp
			if (start(1).eq.maxt) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=maxt  
			   polygon(countcoorpol(countpol),countpol)=maxp
			   if (start(2).eq.maxp) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=mint  
			     polygon(countcoorpol(countpol),countpol)=maxp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
			 endif
			 exit seg_loop2          
         elseif (p(1,bb).eq.minp.and.t(1,bb).gt.actual(1).or. &
		 &       p(numcoor(bb),bb).eq.minp.and.t(numcoor(bb),bb).gt.actual(1)) then !has to continue on tmin
           do cc=1,maxseg    !check that other side is empty of segments
		      if (p(1,cc).eq.minp.and.t(1,cc).lt.actual(1).or.&
    		 &  p(numcoor(cc),cc).eq.minp.and.t(numcoor(cc),cc).lt.actual(1) ) then			
                   found =0   !abort polygon
				   return
         	  else
				   found=1			  
			  endif
            enddo  
			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=mint  
			polygon(countcoorpol(countpol),countpol)=minp
			if (start(1).eq.mint) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=mint  
			   polygon(countcoorpol(countpol),countpol)=maxp
			   if (start(2).eq.maxp) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=maxt  
			     polygon(countcoorpol(countpol),countpol)=maxp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
			 endif
			 exit seg_loop2 		 
		 endif  !(se == significa che e lo stesso segmento)
   enddo seg_loop2
elseif (actual(2).eq.maxp) then

   seg_loop3: do bb=1,maxseg
      if (maxseg==1) stop 'calculations with only one segment cannot be treated'
	     if (p(1,bb).eq.maxp.and.t(1,bb).lt.actual(1).or.&
		 &  p(numcoor(bb),bb).eq.maxp.and.t(numcoor(bb),bb).lt.actual(1) ) then !has to continue on maxt
            do cc=1,maxseg    !check that other side is empty of segments
		      if (p(1,cc).eq.maxp.and.t(1,cc).gt.actual(1).or.&
		 &  p(numcoor(cc),cc).eq.maxp.and.t(numcoor(cc),cc).gt.actual(1) )  then			
                   found =0   !abort polygon
				   return
         	  else
				   found=1			  
			  endif
            enddo  
			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=maxt  
			polygon(countcoorpol(countpol),countpol)=maxp
			if (start(1).eq.maxt) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=maxt  
			   polygon(countcoorpol(countpol),countpol)=minp
			   if (start(2).eq.minp) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=mint  
			     polygon(countcoorpol(countpol),countpol)=minp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
			 endif
			 exit seg_loop3          
         elseif (p(1,bb).eq.maxp.and.t(1,bb).gt.actual(1).or. &
		 &       p(numcoor(bb),bb).eq.maxp.and.t(numcoor(bb),bb).gt.actual(1)) then !has to continue on tmin
            do cc=1,maxseg    !check that other side is empty of segments
		      if (p(1,cc).eq.maxp.and.t(1,cc).lt.actual(1).or.&
		 &  p(numcoor(cc),cc).eq.maxp.and.t(numcoor(cc),cc).lt.actual(1) )  then			
                   found =0   !abort polygon
				   return
         	  else
				   found=1			  
			  endif
            enddo
			countcoorpol(countpol)=countcoorpol(countpol)+2
			polygon(countcoorpol(countpol)-1,countpol)=mint  
			polygon(countcoorpol(countpol),countpol)=maxp
			if (start(1).eq.mint) then !polygon can be finished
			   countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			   polygon(countcoorpol(countpol),countpol)=start(2)
			else !polygon has to go rounda 2 corner
               countcoorpol(countpol)=countcoorpol(countpol)+2
			   polygon(countcoorpol(countpol)-1,countpol)=mint  
			   polygon(countcoorpol(countpol),countpol)=minp
			   if (start(2).eq.minp) then !polygon can be finished
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   else !polygon has to go rounda 3 corner
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=maxt  
			     polygon(countcoorpol(countpol),countpol)=minp
			     countcoorpol(countpol)=countcoorpol(countpol)+2
			     polygon(countcoorpol(countpol)-1,countpol)=start(1)  
			     polygon(countcoorpol(countpol),countpol)=start(2)
			   endif
			 endif
			 exit seg_loop3 		 
		 endif  !(se == significa che e lo stesso segmento)
   enddo seg_loop3


endif
end subroutine onesegpolygon
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------
endmodule build_poly
!************************************************************************************
!************************************************************************************
!************************************************************************************
!MAIN
program Polygons
!--------------------------------------------------------------------------------------
!this program takes as imput file a plot file produced by vertex and converts it
!into a digitalized phase diagram section. 
use input_output
use geom_lib
use build_poly


implicit none
!--------------------------------------------------------------------------------------
call read_input        !reading input and prepering output file
call build             !build polygons
call output 

end program Polygons

